/*
* Light_WS2812 library example - RGB_blinky
*
* cycles one LED through red, green, blue
*
* This example is configured for a ATtiny85 with PLL clock fuse set and
* the WS2812 string connected to PB4.
*/
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "light_ws2812.h"


#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define bit_delay_time 102 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay


#define input_port PORTA
#define input_direction DDRA
#define btn_pin (1 << PA7)
#define input_pins PINA

void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   //
   // char delay
   //
   bit_delay();
   }

struct cRGB led[8];

int main(void) {
   //
   // main
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // initialize pins
   //

   set(input_port, btn_pin); // turn on pull-up
   input(input_direction, btn_pin);
   //
   // main loop
   //
   while (1) {
      
      //
      // wait for button down
      //
      while (0 == pin_test(input_pins, btn_pin)) {
         int i = 255;
         for (i = 255; i>= 0; i--){
            led[0].r=i;led[0].g=i;led[0].b=i;    // Write red to array
            led[1].r=i;led[1].g=i;led[1].b=i;        // green
            led[2].r=i;led[2].g=i;led[2].b=i;
            led[3].r=i;led[3].g=i;led[3].b=i;
            led[4].r=i;led[4].g=i;led[4].b=i;
            led[5].r=i;led[5].g=i;led[5].b=i;
            led[6].r=i;led[6].g=i;led[6].b=i;
            led[7].r=i;led[7].g=i;led[7].b=i;
            ws2812_setleds(led,8);
            _delay_ms(117);}
      }   
      //
      // wait for button up
      //
      while (0 != pin_test(input_pins, btn_pin)) {
            led[0].r=0;led[0].g=0;led[0].b=0;    // Write red to array
            led[1].r=0;led[1].g=0;led[1].b=0;        // green
            led[2].r=0;led[2].g=0;led[2].b=0;
            led[3].r=0;led[3].g=0;led[3].b=0;
            led[4].r=0;led[4].g=0;led[4].b=0;
            led[5].r=0;led[5].g=0;led[5].b=0;
            led[6].r=0;led[6].g=0;led[6].b=0;
            led[7].r=0;led[7].g=0;led[7].b=0;
            ws2812_setleds(led,8);
            _delay_ms(10);     
      }
   }
}



